/************************************************/
/* Created by Renesas Technology America, Inc.  */
/* Makers of M16C, H8, SH                       */
/* http://www.renesas.com                       */
/************************************************/

/* tab space = 4 */
#include "file_system.h"


/* include web files */
#include "index_html.h"
#include "lcd_set_html.h"
#include "http404_html.h"
#include "logo_gif.h"
#include "marble_jpg.h"

#define LCD_TEXT_L1 623
#define LCD_TEXT_L2 643

extern char lcd_text1[17],lcd_text2[17];

/* Allocate the space for the files */
file_entry FAT[MAX_FILES+1] = 
{
	/*** Entry 0 ***/
	{0,0,0,0},		/* skip the first entry becuase fstart can't be 0 */
	/*** Entry 1 ***/
					/* must always be the the HTTP 404 file that will returned */
	{"http404.html",http404_html_SIZE,
		{(UINT32)http404_html},
		{http404_html_SIZE}
	},
	/*** Entry 2 ***/
	{"index.html",index_html_SIZE,
		{
		(UINT32)index_html,
		(UINT32)lcd_text1,
		(UINT32)(index_html + index_html_TAG_2),
		(UINT32)lcd_text2,
		(UINT32)(index_html + index_html_TAG_4),
		},
		{
		index_html_TAG_1 - 0,
		index_html_TAG_2 - index_html_TAG_1,
		index_html_TAG_3 - index_html_TAG_2,
		index_html_TAG_4 - index_html_TAG_3,
		index_html_SIZE - index_html_TAG_4,
		}
	},

	/*** Entry 3 ***/
	{"logo.gif",logo_gif_SIZE,
		{(UINT32)logo_gif},
		{logo_gif_SIZE}
	},
	/*** Entry 4 ***/
	{"lcd_set.html",lcd_set_html_SIZE,
		{(UINT32)lcd_set_html},
		{lcd_set_html_SIZE}
	},
	/*** Entry 5 ***/
	{"marble.jpg",marble_jpg_SIZE,
		{(UINT32)marble_jpg},
		{marble_jpg_SIZE}
	},

};


INT8 FindFile(UINT32 name_str)
{
	UINT8 i;
	FAR char * str1, * str2;

	for(i=0;i<MAX_FILES+1;i++)
	{
		// Compare the strings 
		str1 = (FAR char *) name_str;
		str2 = (FAR char *) FAT[i].name;

		while(*str1 == *str2)
		{
			/* if both at the null, then strings match */
			if( *str1 == 0)
				return i;
			else
			{
				str1++;
				str2++;
			}
		}
	}

	/* not found */
	return -1;
}
UINT16 GetFileSize( UINT8 fat_index )
{
	return FAT[fat_index].size;
}


/* Returns the number of bytes read */
UINT16 ReadFile(UINT8 fat_index, UINT16 offset, UINT8 * buffer, UINT16 bytes2read)
{
	UINT16 i,sector_end;
	UINT32 copy_from;
	UINT16 bytes_read = bytes2read;

	/* First find the sector in which the offset starts in, then copy
	/* out the data. If we are at the end of a sector and we still
	/* need more data, then start reading from the next sector */
	while( bytes2read)
	{
		/* Find the sector begining with the offset */
		sector_end = 0;
		copy_from = 0;
		for(i=0;i<MAX_FILES_SECTORS;i++)
		{
			sector_end += FAT[fat_index].sector_data_size[i];
			if(offset < sector_end )
				break;		// found sector to start reading from

			copy_from = sector_end;		// save this so we know how far into the file the sector will starts
		}

		/* check for end of file (ie, ran out of data to read) */
		if( i == MAX_FILES_SECTORS)
		{
			bytes_read -= bytes2read;		// return the bytes we still had left to read.
			break;
		}

		/* Get the address location from where we will start reading from */
		copy_from = offset - copy_from;		// this will give use the offset from the begining of the sector
		copy_from += FAT[fat_index].sector_data[i];		// add in the memory location of the begining of the sector

		/* Find out how much we can copy */
		if( offset + bytes2read > sector_end)
		{
			// we can only copy to the end of the sector on this pass
			i = sector_end - offset;
		}
		else
		{
			/* All the rest of the data we need is in this sector */
			i = bytes2read;
		}

		bytes2read -= i;	// decrease count of bytes left to read
		offset += i;		// changed the file offset if we need a nother pass in this loop

		/* block copy data from this sector */
		while(i--)
			*(UINT8 *)buffer++ = *(FAR UINT8 *)copy_from++;
	}

	return bytes_read;
}