#ifndef LAN91C96_H
#define LAN91C96_H

#define MIN_PACKET_SIZE    64

/* Header file for SMSC91C96 Definitions
   SYS TEC Computer GmbH Oktober 1999
   M. Berthel (m.b.)
*/

#define BANK0   0x0000
#define BANK1   0x0001
#define BANK2   0x0002
#define BANK3   0x0003
#define BANK4   0x0004
#define BANK5   0x0005
#define BANK6   0x0006
#define BANK7   0x0007

#define REG_BANKSEL   (0x0e)
#define REG_TCR       (0x00)       /* bank 0 registers */
#define REG_STATUS    (0x02)
#define REG_EPHSRS    (0x02)
#define REG_RCR       (0x04)
#define REG_COUNTER   (0x06)
#define REG_MIR       (0x08)
#define REG_MCR       (0x0a)
#define REG_TEST      (0x0c)
#define REG_CR        (0x00)       /* bank 1 registers */
#define REG_BASE      (0x02)
#define REG_IAR       (0x04)
#define REG_IA10      (0x04)
#define REG_IA32      (0x06)
#define REG_IA54      (0x08)
#define REG_GENERAL   (0x0a)
#define REG_CTR       (0x0c)
#define REG_MMUCR     (0x00)       /* bank 2 registers */
#define REG_PNR       (0x02)
#define REG_ARR       (0x03)
#define REG_FIFO      (0x04)
#define REG_PTR       (0x06)
#define REG_DATA1     (0x08)
#define REG_DATA2     (0x0a)
#define REG_INTERRUPT (0x0c)
#define REG_IST       (0x0c)
#define REG_ACK       (0x0c)
#define REG_INTR_MASK (0x0d)
#define REG_MT10      (0x00)       /* bank 3 registers */
#define REG_MT32      (0x02)
#define REG_MT54      (0x04)
#define REG_MT76      (0x06)
#define REG_MGMT      (0x08)
#define REG_REVISION  (0x0a)
#define REG_ERCV      (0x0c)

#define REG_ECOR      (0x00)       /* bank 4 registers */
#define REG_EBASE0    (0x04)
#define REG_EBASE1    (0x06)

#define REG_MCOR      (0x00)       /* bank 5 registers */
#define REG_PRR       (0x02)
#define REG_IOEIR     (0x04)
#define REG_MBASE1    (0x06)
#define REG_MSIZE     (0x08)

#define FDSE       0x8000       /* tcr register bits */
#define ETEN_TYPE  0x4000
#define EPHLOOP    0x2000
#define STPSQET    0x1000
#define FDUPLX     0x0800
#define MONCSN     0x0400
#define NOCRC      0x0100
#define PADEN      0x0080
#define FORCOL     0x0004
#define LOOP       0x0002
#define TXENA      0x0001

/* EPH Status Register Bits (EPHSR) */
#define TX_UNRN     0x8000
#define LINK_OK     0x4000
#define RES         0x2000
#define CTR_ROL     0x1000
#define EXC_DEF     0x0800
#define LOST_CARR   0x0400
#define LAT_COL     0x0200
#define WAKEUP      0x0100
#define TX_DEFR     0x0080
#define LTX_BRD     0x0040
#define SQET        0x0020
#define SIXTEEN_COL 0x0010
#define LTX_MULT    0x0008
#define MUL_COL     0x0004
#define SNGL_COL    0x0002
#define TX_SUC      0x0001

#define SOFTRST    0x8000       /* rcr register bits */
#define FILTCAR    0x4000
#define STRIPCRC   0x0200
#define RXEN       0x0100
#define ALMUL      0x0004
#define PRMS       0x0002
#define RXABORT    0x0001

#define MIISELECT  0x8000      /* configuration register bits */
#define NOWAITST   0x1000
#define FULLSTEP   0x0400
#define SETSQLCH   0X0200
#define AUISELECT  0x0100
#define SIXTEENBIT 0x0080
#define INTSEL     0x0006

#define RCVBAD     0x4000      /* control register bits */
#define PWRDN      0x2000
#define AUTORELEASE 0x0800
#define LEENABLE   0x0080
#define CRENABLE   0x0040
#define TEENABLE   0x0020
#define EEPROMSEL  0x0004
#define RELOAD     0x0002
#define STORE      0x0001

/* MMU Command Register bits (MMUCR) */
#define BUSY       0x0001

#define FAILALL    0x8000      /* allocation result register */

#define REMPTY     0x8000      /* fifoports register bits */
#define TEMPTY     0x0080

/* Pointer Register Bits (PTR) */
#define RCV        0x8000
#define AUTO_INCR  0x4000
#define READ       0x2000
#define ETEN       0x1000
#define AUTOTX     0x0800

/* Interrupt Status Register bits (IST) */
#define TX_IDLE_INT  0x0080
#define ERCVINT      0x0040
#define EPH_INT      0x0020
#define RX_OVRN_INT  0x0010
#define ALLOC_INT    0x0008
#define TX_EMPTY_INT 0x0004
#define TX_INT       0x0002
#define RCV_INT       0x0001

#define ERCVINT_EN    0x4000      /* interrupt mask register bits */
#define EPHINT_EN     0x2000
#define RXOVRNINT_EN  0x1000
#define ALLOCINT_EN   0x0800
#define TXEMPTYINT_EN 0x0400
#define TXINT_EN      0x0200
#define RXINT_EN      0x0100

#define ERCVINTMASK     0x4000   /* interrupt mask register */
#define EPHINTMASK      0x2000
#define RXOVRNINTMASK   0x1000
#define ALLOCINTMASK    0x0800
#define TXEMPTYINTMASK  0x0400
#define TXINTMASK       0x0200
#define RXINTMASK       0x0100

#define ALGNERR    0x8000     /* receive frame status word */
#define BRODCAST   0x4000
#define BADCRC     0x2000
#define ODDFRM     0x1000
#define TOOLONG    0x0800
#define TOOSHORT   0x0400

#define EPHERR     0x8630

#define CLEAR        0x0000
#define FIFO_TX_MASK 0x007f
#define PNR_MASK     0x00FF
#define ARR_MASK     0xFF00

#endif

