/*
 *Copyright (c) 2000-2002 Viola Systems Ltd.
 *All rights reserved.
 *
 *Redistribution and use in source and binary forms, with or without 
 *modification, are permitted provided that the following conditions 
 *are met:
 *
 *1. Redistributions of source code must retain the above copyright 
 *notice, this list of conditions and the following disclaimer.
 *
 *2. Redistributions in binary form must reproduce the above copyright 
 *notice, this list of conditions and the following disclaimer in the 
 *documentation and/or other materials provided with the distribution.
 *
 *3. The end-user documentation included with the redistribution, if 
 *any, must include the following acknowledgment:
 *	"This product includes software developed by Viola 
 *	Systems (http://www.violasystems.com/)."
 *
 *Alternately, this acknowledgment may appear in the software itself, 
 *if and wherever such third-party acknowledgments normally appear.
 *
 *4. The names "OpenTCP" and "Viola Systems" must not be used to 
 *endorse or promote products derived from this software without prior 
 *written permission. For written permission, please contact 
 *opentcp@opentcp.org.
 *
 *5. Products derived from this software may not be called "OpenTCP", 
 *nor may "OpenTCP" appear in their name, without prior written 
 *permission of the Viola Systems Ltd.
 *
 *THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED 
 *WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
 *MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 *IN NO EVENT SHALL VIOLA SYSTEMS LTD. OR ITS CONTRIBUTORS BE LIABLE 
 *FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 *CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 *SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
 *BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 *OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
 *EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *====================================================================
 *
 *OpenTCP is the unified open source TCP/IP stack available on a series 
 *of 8/16-bit microcontrollers, please see <http://www.opentcp.org>.
 *
 *For more information on how to network-enable your devices, or how to 
 *obtain commercial technical support for OpenTCP, please see 
 *<http://www.violasystems.com/>.
 */

/** \file ethernet_ks884x.c
 *	\brief OpenTCP Ethernet protocol and driver for Micrel KS8841/KS8842.
 *	\author 
 *		\PCD (support@micrel.com)
 *	\version 0.1.0
 *	\date 03/30/2005
 *	\bug
 *	\warning
 *	\todo
 *  
 *	OpenTCP implementation of Ethernet protocol and driver 
 */

#include <inet/system.h>

/************************************************/
/* Modified by Renesas Technology America, Inc. */
/* Makers of M16C, H8, SH                       */
/* http://www.renesas.com                       */
/************************************************/

#include <string.h>
#include "hardware.h"
#include "hw_ver.h"
#include "ks_ver.h"
#include "ks_ver_put.h"
#include "ks_ver_qc.h"


#define DRV_DEBUG
/*
 */
#ifdef DRV_DEBUG        /* if debugging driver */
#    define DRV_DEBUG_OFF           0x0000
#    define DRV_DEBUG_ERROR         0x0001
#    define DRV_DEBUG_RX            0x0010
#    define DRV_DEBUG_TX            0x0020
#    define DRV_DEBUG_INT           0x0040
#    define DRV_DEBUG_CHIP_INIT     0x0080
#    define DRV_DEBUG_ALL           0xffff

static UINT16 ksDebug = ( DRV_DEBUG_ERROR /*| DRV_DEBUG_RX | DRV_DEBUG_INT | DRV_DEBUG_TX  */);

#define DRV_PRINT(FLG,X)                                                \
	if (ksDebug & FLG) DBG_PRINT X;

#else /* DRV_DEBUG */

#define DRV_PRINT(FLG,X)                                                \

#endif /* DRV_DEBUG */



extern int hwwrite ( unsigned char, unsigned long, unsigned long, unsigned char );
extern int hwread ( unsigned char, unsigned long, unsigned char );

/************ Configurations ************/

#if M16C_62P
#define BASE_ADDR 0x10300
#endif

#undef     BRINGUP_BOARD          /* Just to bring up the demo board, without init the device */


#define INTERRUPT_ETHERNET 1     /* Receive packet from device by interrupt */
#define POLLED_ETHERNET    0     /* Receive packet from device by polling   */


//#define RX_BUF_SIZE 1550

/* For our transmit buffer, use the same size that we specified in system.h */
//#define TX_BUF_SIZE NETWORK_TX_BUFFER_SIZE


#define MSB(x)	(((x) >> 8) & 0xff)	  /* most signif byte of 2-byte integer */
#define LSB(x)	((x) & 0xff)		  /* least signif byte of 2-byte integer*/

/* swap the MSB with the LSB of a 16 bit integer */
#define WORDSWAP(x) (MSB(x) | (LSB(x) << 8))

#define LLSB(x)	((x) & 0xff)		/* 32bit word byte/word swap macros */
#define LNLSB(x) (((x) >> 8) & 0xff)
#define LNMSB(x) (((x) >> 16) & 0xff)
#define LMSB(x)	 (((x) >> 24) & 0xff)
#define LONGSWAP(x) ((LLSB(x) << 24) | \
		     (LNLSB(x) << 16)| \
		     (LNMSB(x) << 8) | \
		     (LMSB(x)))

#define SYS_INT_ENABLE   \
	int2ic = 0x04 		/* enable INT2 pin, falling edge triggered, priority level 4 */	

const char *deviceName[] =
{
/*  0 */  "UNKNOWN",
/*  1 */  "KSZ8841",
/*  2 */  "KSZ8842",
/*  3 */  "KSZ8861",
/*  4 */  "KSZ8862",
	      ""
};

BYTE 	fKS884x = 0;     
 
UINT8	NE2000NextPktPtr;			/**< Start address of next packet in the Ethernet controller */
UINT8 	NE2000CurrPktPtr;			/**< Start address of current packet in the Ethernet controller */

UINT8 	EtherSleep = 0;	/**< Used for storing state of Ethernet controller (0 = awake; 1 = sleeping) */

UINT8   rx_buffer_filled = 0;	/* used to signal when a Rx interrupt has occured and the buffer has been filled */


/* allocate tx/rx buffers */
UINT8 eth_pkt_rx_buf[RX_BUF_SIZE];
UINT8 * eth_rx_buf_ptr;
UINT8 eth_pkt_tx_buf[TX_BUF_SIZE];
UINT8 * eth_tx_buf_ptr;

UINT8 eth_pkt_hdr_rx_buf[ETH_HEADER_LEN];

/* for debug use */
UINT8   eth_pkt_rx_buf_temp[RX_BUF_SIZE+ETH_HEADER_LEN];

struct _ETH_INFO
{
	UINT32 total_rx_packets;
	UINT32 total_tx_packets;
	UINT8 full_duplex:1;
	UINT8 speed_100:1;
	UINT8 linked:1;
} eth_info = {0,0,0,0,0};

/** \brief Used for storing various information about the received Ethernet frame
 *	
 *	Fields from Ethernet packet (dest/source hardware address, 
 *	protocol, frame size, start of the Ethernet packet in Ethernet controller)
 *	are stored in this structure. These values are later used from upper
 *	layer protocols (IP, ARP). See ethernet_frame definition for more
 *	information about struct fields.
 *
 */	
struct ethernet_frame received_frame;

/** \brief Used for storing various information about the Ethernet frame that will be sent
 *	
 *	Fields from Ethernet packet (dest/source hardware address, 
 *	protocol, frame size) are stored in this structure by the
 *	upper layer protocols (IP, ARP, other). These values are 
 *	then used for initializing transmission of an Ethernet frame.
 *	See ethernet_frame definition for more
 *	information about struct fields.
 *
 */
struct ethernet_frame send_frame;


/** \brief ks884x chip hardware structure
 *	
 */
HARDWARE   hw;
PHARDWARE  phw = &hw;

/** \brief ks884x API functions 
 *	
 */
struct hw_fn  ks_fn;
struct hw_fn *pks_fn=&ks_fn;



void  ks884xIntr(void);
UINT8 ksReceiveEarlyFrame (void);
UINT8 ksReceiveFrame (void);
UINT8 ksSendFrame ( UINT8 *, UINT16, ULONG );

static UINT8 ksEnetAddrSet ( char * );
static void  showKsBspInfo ( void );
static void  showKsLinkStatus ( void );
static BOOLEAN QCCheckChipID ( PHARDWARE );


/* Functions not used in this implementation */
//void outNE2000 (UINT8 reg , UINT8 dat) { }	
//UINT8 inNE2000 (UINT8 reg) { }
//void NE2000CheckOverFlow (void) { }
//void NE2000DMAInit (UINT8 page) { }
//void NE2000EnterSleep (void) { }
//void NE2000ExitSleep (void) { }



void hw_read_dword
(   
    PHARDWARE phwi, 
    UCHAR     addr, 
    ULONG    *data 
)                                   
{                                                                           
    USHORT  wData, wDumy;                                                          
    ULONG   dwData;
    HW_READ_WORD( phwi, (UCHAR)(addr), (USHORT *)&wData ); 
    wDumy = wData;
    HW_READ_WORD( phwi, (UCHAR)(addr+2), (USHORT *)&dwData );                  
    dwData <<= 16;                                                          
    dwData |= (ULONG)wData;                                                 
    *(ULONG *)data = dwData;                                                
}

void hw_write_dword
(   
    PHARDWARE phwi, 
    UCHAR     addr, 
    ULONG     data 
)                                   
{                                                                           
    USHORT  wData, wDumy;                                                          
    wData = (USHORT)(data & 0x0000ffff);                                    
    HW_WRITE_WORD( phwi, (UCHAR)addr, (USHORT)wData );                      
    wDumy = wData;
    wData = (USHORT)( (data & 0xffff0000) >> 16 );                          
    HW_WRITE_WORD( phwi, (UCHAR)(addr+2), (USHORT)wData );     
}



/** \brief Write data to the same NE2000 register as before
 * 	\author 
 *		\li Jari Lahti (jari.lahti@violasystems.com)
 *	\date 19.02.2002
 *	\param dat new register content
 *
 *	Invoke this function to write data to the same register as 
 *	in previously invoked outNE2000(). Saves some cycles because
 *	data direction registers and address are not set up.
 *
 *	Use SEND_NETWORK_B() macro instead of invoking this function
 *	directly.
 */
void outNE2000again (UINT8 dat)
{
	/* Actually, because this function wants to write a byte and our */
	/* Ethernet controller only wants to write words, we will write
	/* every thing to a buffer, then send it later */
	*eth_tx_buf_ptr++ = dat;
}
/** \brief Write buffer data to the same NE2000 register as before
 * 	\author 
 *		\li Jari Lahti (jari.lahti@violasystems.com)
 *	\date 20.07.2003
 *	\param buf pointer to buffer from which we're taking data
 *	\param len number of bytes from buffer to write
 *
 *	Use SEND_NETWORK_BUF() macro instead of invoking this function
 *	directly.
 */
void outNE2000againbuf (UINT8* buf, UINT16 len)
{
	while(len--)
	{
		*eth_tx_buf_ptr++ = *buf++;
	}
}

/** \brief Continue reading byte(s) from NE2000
 * 	\author 
 *		\li Jari Lahti (jari.lahti@violasystems.com)
 *	\date 19.02.2002
 *	\return Register content
 *
 *	Invoke this function to continue reading contents of a NE2000 register,
 *	usefull when reading a sequence of bytes from Ethernet controller.
 *
 *	Use RECEIVE_NETWORK_B() macro instead of directly invoking this
 *	function for easier driver change.
 */
UINT8 inNE2000again (void)
{
	/* Actually, because this function wants to read a byte and our */
	/* Ethernet controller only wants to read words, we will read
	/* every thing from a buffer that we filled already */
	return 	*eth_rx_buf_ptr++;
}
/** \brief Read bytes from NE2000 to buffer
 * 	\author 
 *		\li Jari Lahti (jari.lahti@violasystems.com)
 *	\date 20.07.2003
 *
 *	Invoke this function to continue reading contents of a NE2000 register,
 *	only this time to a buffer directly.
 *
 *	Use RECEIVE_NETWORK_BUF() macro instead of directly invoking this
 *	function for easier driver change.
 */
void inNE2000againbuf (UINT8* buf, UINT16 len)
{
	while(len--)
	{
		*buf++ = *eth_rx_buf_ptr++;
	}
}


/** \brief Check to see if new frame has been received
 * 	\author 
 *		\li Jari Lahti (jari.lahti@violasystems.com)
 *	\date 19.02.2002
 *	\return
 *		\li #TRUE - if there is a new packet waiting
 *		\li #FALSE - if there is no new packet waiting
 *
 *	Invoke this function to check if there is a new Ethernet frame
 *	in the NIC's buffer waiting to be processed.
 */
UINT8 NE2000CheckRxFrame (void)
{
    UINT16  wIntStatus;
    UINT8   bRetStatus=FALSE;
    UINT16  wMorePacketLength=0;
    UINT8   portIndex=0;


#ifndef BRINGUP_BOARD
	/* Read Interrupt Status Register to determine if a new packet has arrived */
    HardwareReadInterrupt( phw, &wIntStatus );

	/* Check if a receive error has occured */
	if( wIntStatus & INT_RX_OVERRUN)
	{
        phw->m_cnCounter[ portIndex ][ OID_COUNTER_RCV_NO_BUFFER ]++;

        /* Clear the interrupt INT_RX_OVERRUN status */
        HardwareAcknowledgeOverrun( phw );

	    DRV_PRINT(DRV_DEBUG_RX, 
              ("NE2000CheckRxFrame(): Receive OverRun error!"NEWLINE));
	}

	if( wIntStatus & INT_RX_ERROR)
	{
        phw->m_cnCounter[ portIndex ][ OID_COUNTER_RCV_INT_ERROR ]++;

        /* Clear the interrupt INT_RX_ERROR status */
        HardwareAcknowledgeErrorFrame( phw );

	    DRV_PRINT(DRV_DEBUG_ERROR, 
              ("NE2000CheckRxFrame(): Receive Error Frame!"NEWLINE));
	}

	if( wIntStatus & INT_RX_STOPPED)
	{
        phw->m_cnCounter[ portIndex ][ OID_COUNTER_RCV_INT_STOP ]++;

        /* Clear the interrupt INT_RX_STOPPED status */
        HardwareAcknowledgeRxStop( phw );

	    DRV_PRINT(DRV_DEBUG_ERROR, 
              ("NE2000CheckRxFrame(): Receive Process Stopped!"NEWLINE));
	}


    /* Check if a new packet has arrived of more packets are in the RXQ */
	if ( 
         (wIntStatus & INT_RX) &&
         ( (wMorePacketLength = HardwareReceiveMoreDataAvailable ( phw ) )  > 0 )
       )
	{
		/* At least one packet is available in the RXQ. */
        bRetStatus = TRUE;
	}

#endif

	return(bRetStatus);
}

/** \brief Discard current received frame
 * 	\author 
 *		\li Jari Lahti (jari.lahti@violasystems.com)
 *	\date 19.02.2002
 *
 *	Discard the current received frame.
 */
void NE2000DumpRxFrame (void)
{
	rx_buffer_filled = 0;	// allow the next packet to come in
}
 

/** \brief Initialize and configure Ethernet
 *	\ingroup core_initializer
 * 	\author 
 *		\li Jari Lahti (jari.lahti@violasystems.com)
 *	\date 19.02.2002
 *	\param mac Pointer to NIC's hardware address
 *
 *	Invoke this function at startup to properly initialize NIC's registers,
 *	HW address and operation.
 */ 
void NE2000Init (UINT8* mac)
{
    UINT8    i;
    USHORT   ChipID;
    UCHAR    DevRevisionID;
    unsigned char chipBaseAddr[4];


    memset( phw, 0, sizeof( HARDWARE ));
    memset( pks_fn, 0, sizeof( struct hw_fn ));

    /* Set ks884x base memory map address */
    phw->m_ulVIoAddr = BASE_ADDR;

    /* Set ks884x internal API function point */
    HardwareSetupFunc( pks_fn );
    phw->m_hwfn = pks_fn;

    phw->m_bPromiscuous = FALSE;
    phw->m_bAllMulticast = TRUE;
    phw->m_bMulticastListSize = 0;

    phw->fDebugDumpTx = FALSE;
    phw->fDebugDumpRx = FALSE;

#ifndef BRINGUP_BOARD  

	/* Reset Chip, and check Chip ID */
    if (
        !HardwareReset( phw ) ||        
#ifndef KS_QC_TEST
        !HardwareInitialize( phw ) ) 
#else
        !QCCheckChipID( phw ) ) 
#endif
    {
		DisplayString(LCD_LINE1,"CHIP    ");
		DisplayString(LCD_LINE2,"error...");
	    DRV_PRINT(DRV_DEBUG_ERROR, 
                  ("CHIP ID error...   "NEWLINE));
        HardwareReadChipID ( phw, &ChipID, &DevRevisionID );
        DBG_PRINT("Current Chip ID is %04x", ChipID);   
        DBG_PRINT( NEWLINE );
		while(1);		/* ERROR 2 - chip error, Stop here */
    }

    /* Get ks884x station MAC address */
    HardwareReadAddress( phw );

	/* if device's MAC address is different than driver's MAC address */
   	if (phw->m_bPermanentAddress[0] != mac[0] || 
        phw->m_bPermanentAddress[1] != mac[1] ||
		phw->m_bPermanentAddress[2] != mac[2] || 
        phw->m_bPermanentAddress[3] != mac[3] ||
		phw->m_bPermanentAddress[4] != mac[4] || 
        phw->m_bPermanentAddress[5] != mac[5]) 

    {
        /* Set driver's default MAC to ks884x station MAC address */
	    DRV_PRINT(DRV_DEBUG_RX, 
              ("NE2000Init(): Device MAC is different than driver MAC, set driver MAC to device."NEWLINE));

        /* 
         * Set Device's MAC address.
	     * ADDRESS 0 corresponds to the first address byte on the cable 
	     * OpenTCP defines the localHW[6] array as [5]:[4]:[3]:[2]:[1]:[0] 
         */   
        if ( ksEnetAddrSet( (char *) mac) == FALSE )
        {
	        DRV_PRINT(DRV_DEBUG_ERROR, 
                     ("NE2000Init(): fail to set MAC address to device."NEWLINE));
        }
    }

#ifdef DEF_KS8842
    /* Set Switch function address */
    SwitchSetAddress ( phw, phw->m_bOverrideAddress );
#endif

    HardwareSetGroupAddress( phw );

    /* Clear MIB's Counters */
    phw->m_bPortSelect = 0;         /* clear Port 1 */
    HardwareClearCounters( phw );
#ifdef DEF_KS8842
    phw->m_bPortSelect = 1;         /* clear Port 2 */
    HardwareClearCounters( phw );
#endif
    phw->m_bPortSelect = 2;         /* clear Port 3 */
    HardwareClearCounters( phw );

    /* Configure Chip */ 
    phw->m_bPortTX = 0;
    HardwareSetup( phw );

    /* Enables the hardware. */
    HardwareEnable( phw );

#ifndef KS_QC_TEST
    HardwareSwitchSetup ( phw );

    /* Wait for auto negotiation to complete. */
    DelayMillisec( 2000 );

#endif /* #ifndef KS_QC_TEST */

#ifdef DEF_KS8842
    /* Enable Switch */
    SwitchEnable ( phw, TRUE );
#endif


#if (INTERRUPT_ETHERNET)

    /* Initialise the hardware interrupt mask for proper operation. */
    HardwareSetupInterrupt( phw );

#if defined (TEST_THROUGHPUT) && !defined (EARLY_TRANSMIT)
    /* disable TX interrupt */
    phw->m_wInterruptMask &= ~((UINT16)INT_TX);  
#endif

    /* Clear the interrupts status of the hardware. */
    HardwareAcknowledgeInterrupt( phw, 0xFFFF );

    /* Enables the interrupts of the hardware. */
    HardwareEnableInterrupt( phw ); 

	/* enable interrupts for M16C/62P */
    SYS_INT_ENABLE; 

#endif

    /* Get Device's MAC address */
    HardwareReadAddress( phw );

    /* Get Port Link Status */
    SwitchGetLinkStatus( phw );

#endif  /* BRINGUP_BOARD */

    /* Display Device Driver info on UART */
    showKsBspInfo ();

    /* Display Ethernet Link status on UART */
    showKsLinkStatus ();

    /* Clear PMEN pin */
    HardwareClearWolPMEStatus ( phw );

   /* Wake-on-LAN LED off */
    wol_lcd_off();

}

/** \brief Checks if new Ethernet frame exists and initializes variables
 *		accordingly
 * 	\author 
 *		\li Jari Lahti (jari.lahti@violasytems.com)
 *	\date 19.02.2002
 *	\return
 *		\li #TRUE - new frame exists, variables initialized so further
 *			processing may start
 *		\li #FALSE - no new Ethernet frame detected
 *
 *	Invoke this function periodically (better not directly but by invoking
 *	NETWORK_CHECK_IF_RECEIVED() macro) to test for newly-arrived Ethernet
 *	packets.
 *
 *	Once this function detects a new Etherent packet, internal variables 
 *	are initialized (NE2000CurrPktPtr, NE2000NextPktPtr and received_frame
 *	structure) and TRUE is returned, indicating that processing of
 *	newly-arrived frame may begin.
 */
UINT8 NE2000ReceiveFrame (void)
{
    int     packet_len = 0;
    int     port = 0, portIndex = 0;
    UINT16 *pProtocol;
    UINT8   i, *daAddr, *saAddr;
    USHORT  rxCntl;
    USHORT  rxLength;
    UINT8   status=FALSE;



#ifndef BRINGUP_BOARD

#ifdef TEST_THROUGHPUT
    /* isolate from OpenTCP stack */
	return(status);
#endif


#if (INTERRUPT_ETHERNET)


    if(rx_buffer_filled == 0)
    {
	   if( NE2000CheckRxFrame() == FALSE )
       {
          return (FALSE);		/* no new packets yet */
       }
    }

    if(rx_buffer_filled == 1)
    {
       return (TRUE);		    /* buffer was filled by interrupt routine */
    }

    /* if this far, then interrupt routine wants the buffer to get filled */
    rx_buffer_filled = 0;		// reset flag in case we get error filling buffer
#endif

#if (POLLED_ETHERNET)
    if( NE2000CheckRxFrame() == FALSE ) 
 	    return(FALSE);
#endif

    /* while loop reading data from hardware packet memory, until packet memory is empty */
    while  ( 
              HardwareReceiveMoreDataAvailable ( phw ) > 0  
           )
    {


       /* Checking Received packet status */
       if ( HardwareReceiveStatus( phw, &rxCntl, &rxLength)  )
       {

           /*
            *  Receive Valid and Good Frame.
            */

           /* Get received packet length from hardware packet memory */
           packet_len = HardwareReceiveLength( phw );

           /* update the received port */
           port = phw->m_bPortRX;
           portIndex = phw->m_bPortRX;
           if ( portIndex > 0 )
               portIndex -- ;


           /* 
            *  Packet must be checked with no errors, AND 
            *  the packet data size can't be bigger than our receive buffer. 
            */
           if ( (packet_len > 0) && (packet_len <= RX_BUF_SIZE) )
           {
               /*  received Packet with no errors  */

               /* Read the received data from hardware packet memory to receive buffer */
               HardwareReceiveBuffer( phw, eth_pkt_rx_buf_temp, packet_len );

               received_frame.frame_size = packet_len;		/* save full size of frame */

               /* Record destination Ethernet Address	*/
               daAddr = &eth_pkt_rx_buf_temp[0];
               for (i=0; i<MAC_ADDRESS_LENGTH; i++)
                   received_frame.destination[MAC_ADDRESS_LENGTH-1-i] = daAddr[i];

               /* Record senders Ethernet address 	*/
               saAddr = &eth_pkt_rx_buf_temp[6];
               for (i=0; i<MAC_ADDRESS_LENGTH; i++)
                   received_frame.source[MAC_ADDRESS_LENGTH-1-i] = saAddr[i];

               /* Record Protocol (account for ethernet chip being little endian) */
               pProtocol = (UINT16 *)&eth_pkt_rx_buf_temp[12];
               received_frame.protocol = (UINT16) WORDSWAP (*pProtocol);

               /* Copy to eth_pkt_rx_buf */
               eth_rx_buf_ptr = eth_pkt_rx_buf;
               memcpy ((char FAR *)eth_rx_buf_ptr, 
                       (char FAR *)&eth_pkt_rx_buf_temp[ETH_HEADER_LEN], 
                       (packet_len - ETH_HEADER_LEN)    /* Ethernet header will not be stored in buffer */
                      );

               /* Give the next layer data start buffer index from the start	*/
               received_frame.buf_index = 0;
 	
               eth_info.total_rx_packets++;

               rx_buffer_filled = 1; 	/* signal that the buffer has been filled */

               /* Update receive statistics. */
               phw->m_cnCounter[ portIndex ][ OID_COUNTER_DIRECTED_FRAMES_RCV ]++;
               phw->m_cnCounter[ portIndex ][ OID_COUNTER_DIRECTED_BYTES_RCV ] += packet_len;

               status = TRUE;

               ETH_DEBUGOUT("Ethernet Frame Received\n\r");
           } 
           else
           {
               /*  received Packet length errors  */
	           DRV_PRINT(DRV_DEBUG_ERROR, 
                   ("NE2000ReceiveFrame(): Receive packet length errors! ( length=%d, total=%d,rxCntl=0x%04x, rxLength=0x%04x )"NEWLINE, 
                    (int)packet_len, (int)(phw->m_cnCounter[ portIndex ][ OID_COUNTER_DIRECTED_FRAMES_RCV ]),(int)rxCntl, (int)rxLength ));

           } /* if ( (packet_len > 0) && (packet_len <= RX_BUF_SIZE) ) */
       }
       else
       {
            /*  received Packet with errors  */
	        DRV_PRINT(DRV_DEBUG_ERROR, 
                  ("NE2000ReceiveFrame(): Receive packet with errors! ( rxCntl=0x%04x, rxLength=0x%04x)"NEWLINE, 
                    rxCntl, rxLength ));

       } /* if ( HardwareReceiveStatus( phw, &rxCntl, &rxLength)  ) */


	   /* Free up that memory */
       HardwareReleaseReceive( phw );

       /* Acknowledge the RX interrupt. */
       HardwareAcknowledgeReceive( phw );

       /* dump received data to the console port for debug */
       if ( phw->fDebugDumpRx )
       {
           /* for debug use */
           DBG_PRINT( NEWLINE );
           DBG_PRINT("Rx Cntl Word: 0x%04x, Byte Count: 0x%04x"NEWLINE, 
                    (rxCntl),
                    (rxLength) );
           if ( packet_len > 0 )
           {
               PrintPacketData( (char *)eth_pkt_rx_buf_temp, packet_len, port, 2 );
           }
       }

    } /* while  ( HardwareReceiveMoreDataAvailable ( phw ) > 0 ) */

#endif /* BRINGUP_BOARD */

    return(status);			/* Indicate we got packet or not */

}

/** \brief Initialize transmission of new packet
 * 	\author 
 *		\li Jari Lahti (jari.lahti@violasystems.com)
 *	\date 19.02.2002
 *	\param page Address in Ethernet controller where outgoing packet
 *		buffered
 *
 *	This function is used to initialize transmission of an Ethernet 
 *	packet. Packet is created starting from <i>page</i> address.
 *
 *	Do not invoke this function directly, but instead use 
 *	NETWORK_SEND_INITIALIZE() macro.
 */
void InitTransmission (UINT8 page)
{
	/* Simply set up pointer for transmit buffer used by outNE2000again() */
	eth_tx_buf_ptr = eth_pkt_tx_buf;
}

/** \brief Write Ethernet Header to transmission buffer
 * 	\author 
 *		\li Jari Lahti (jari.lahti@violasystems.com)
 *	\date 19.02.2002
 *	\param frame Pointer to ethernet_frame structure containing information
 *		about the new Ethernet frame
 *
 *	Invoke this function (through NETWORK_ADD_DATALINK() macro) to create
 *	an Ethernet header in the NIC's transmit buffer. Do this only after
 *	invoking NETWORK_SEND_INITIALIZE() macro.
 *
 */
void NE2000WriteEthernetHeader (struct ethernet_frame* frame)
{
	INT8 i;
	
	/* Write destination Ethernet Address	*/
	
	for(i=ETH_ADDRESS_LEN-1; i >= 0; i--) {
		outNE2000again(frame->destination[i]);
	}

	/* Write sender (our) Ethernet address	*/
	
	for(i=ETH_ADDRESS_LEN-1; i >= 0; i--) {
		outNE2000again(frame->source[i]);
	}
	
	/* Write protocol						*/
	outNE2000again( (UINT8)(frame->protocol >> 8) );
	outNE2000again( (UINT8)frame->protocol );

}

/** \brief Initialize reading from NE2000
 * 	\author 
 *		\li Jari Lahti (jari.lahti@violasystems.com)
 *	\date 19.02.2002
 *	\param pos Position from the start of current frame from which DMA
 *		is initialized.
 *
 *	Invoke this function (through NETWORK_RECEIVE_INITIALIZE() macro) 
 *	to prepair NIC for reading from the specified position.
 */
void NE2000DMAInit_position (UINT16 pos)
{
	/* Simply set up pointer for receive buffer used by inNE2000again() */
	eth_rx_buf_ptr = eth_pkt_rx_buf + pos;
}

/** \brief Instruct NIC to send the Ethernet frame
 * 	\author 
 *		\li Jari Lahti (jari.lahti@violasystems.com)
 *	\date 19.02.2002
 *	\param dataLength Length of packet stored in the NIC's buffer
 *	\warning
 *		\li Minimum Ethernet packet size is 64 bytes
 *
 *	Invoke this function (through NETWORK_COMPLETE_SEND() macro) when
 *	the whole packet is formed inside NIC's memory and is ready to be 
 *	sent. Proper length of the packet must be supplied so that NIC
 *	knows how much data to put on the line.
 */
void NE2000SendFrame ( UINT16 dataLength )
{
 	/* After filling Tx buffer, call this function to send it */
 	/* Input: 	dataLength: packet length  					  */

    int        control = 0;
    int        port = 0, portIndex = 0;
    int        cnTimeOut = 100;
    UINT8      bIntStatus;
    ULONG      txCntlAndLength=0;
    USHORT     txCntl;
    USHORT     txLength;


#ifndef BRINGUP_BOARD

    /* Set trasnmit destination port */
    if (phw->m_bPortTX > 3)
	    return ;

    port = phw->m_bPortTX;
    portIndex = phw->m_bPortTX;

	/* add Ethernet header */
 	dataLength += ETH_HEADER_LEN;
    
    /* Checking ks8842 TXQ memory available for this packet */
    if ( ! HardwareAllocPacket( phw, dataLength ) ) 
    {
	    DRV_PRINT(DRV_DEBUG_TX, 
              ("NE2000SendFrame(): device TXQ memory not available!"NEWLINE));
	    return;
    }


    /* prepares the transmit 'write' to 'TXQ', write into beginning of ks8842 
       packet memory with 4-byte control information (packet length, and destination port) */
    /* update the received port */

    txCntlAndLength = HardwareSetTransmitLength( phw, dataLength );

	/* Now write the data out to the Ethernet chip */
    /* Continue write packet data (in WORD) into ks8842 packet memory */
    eth_tx_buf_ptr = eth_pkt_tx_buf;
    HW_WRITE_BUFFER( phw, REG_DATA_OFFSET, eth_tx_buf_ptr, dataLength );

    /* Issue TXQ Command (Enqueue Tx frame from TX buffer into TXQ for transmit) */
    if ( HardwareSendPacket( phw ) ) 
    {

        /* Update driver transmit counters */
        if ( portIndex == 3 )
        {
            while ( portIndex & 3)
            {
               portIndex >>= 1;
               phw->m_cnCounter[ portIndex ][ OID_COUNTER_DIRECTED_FRAMES_XMIT ]++;
               phw->m_cnCounter[ portIndex ][ OID_COUNTER_DIRECTED_BYTES_XMIT ] += dataLength;
            }
        }
        else
        {
               if ( portIndex != 0 )
                   portIndex --;
               phw->m_cnCounter[ portIndex ][ OID_COUNTER_DIRECTED_FRAMES_XMIT ]++;
               phw->m_cnCounter[ portIndex ][ OID_COUNTER_DIRECTED_BYTES_XMIT ] += dataLength;
        }

#if (POLLED_ETHERNET)
#ifndef AUTO_RELEASE
        /* wait until device transmit done */
        do {
            HardwareReadInterrupt( phw, &bIntStatus );
        } while ( !(bIntStatus & INT_TX) &&  cnTimeOut-- );

        if ( ( bIntStatus & INT_TX ) )
        {
			/* Transmit successful */
			eth_info.total_tx_packets++;
            HardwareTransmitDone( phw );

            /* Acknowledge the TX interrupt. */
            HardwareAcknowledgeTransmit( phw );
        }
        else
        {
            /* for debug use */
	        DRV_PRINT(DRV_DEBUG_TX, 
                      ("NE2000SendFrame: transmit error happen on device!"NEWLINE));
        }

#endif /* #if AUTO_RELEASE */

#endif /* #if (POLLED_ETHERNET) */

    }

    /* dump transmit data to the console port for debug */
    if ( phw->fDebugDumpTx )
    {
        /* for debug use */
        DBG_PRINT( NEWLINE );
        txCntl   = (txCntlAndLength & 0x0000ffff);
        txLength = (txCntlAndLength >> 16) & 0x0000ffff;
        DBG_PRINT("Tx Cntl Word: 0x%04x, Byte Count: 0x%04x"NEWLINE, 
                    (txCntl),
                    (txLength) );
        PrintPacketData( (char *)eth_pkt_tx_buf, dataLength, port, 1 );
    }

#endif /* #ifndef BRINGUP_BOARD */

}



#if (INTERRUPT_ETHERNET)
#pragma INTERRUPT ks884xIntr
void ks884xIntr(void)
{
    UINT16     wIntStatus;
    int        portIndex = 0;

#ifndef BRINGUP_BOARD

    /* Save address within the TXQ or RXQ frame buffer. */
    HardwareSaveRegs( phw );

    /* Read the device interrupt status register */
    HardwareReadInterrupt( phw, &wIntStatus );

	DRV_PRINT(DRV_DEBUG_INT, 
              ("NEWLINE ks884xIntr(): IntStatus=0x%04X"NEWLINE, wIntStatus));


    /* Only handle the interrupt bits we enable them */
#if (0)
    wIntStatus &= phw->m_wInterruptMask;
#endif
    /*
     * Transmit the Packet Status
     */

    /* Transmitted the packet completed */
    if ( ( wIntStatus & INT_TX ) ) 
    {
		/* Transmit successful */
		eth_info.total_tx_packets++;
        phw->m_cnCounter[ portIndex ][ OID_COUNTER_XMIT_INT ]++;

        HardwareTransmitDone( phw );

        /* Acknowledge the interrupt. */
        HardwareAcknowledgeTransmit( phw );
    }

    /*  Transmit error */
    if ( ( wIntStatus & INT_TX_UNDERRUN ) ) 
    {
        phw->m_cnCounter[ portIndex ][ OID_COUNTER_XMIT_INT_UNDERRUN ]++;

        /* Acknowledge the interrupt. */
        HardwareAcknowledgeUnderrun( phw );

	    DRV_PRINT(DRV_DEBUG_ERROR, 
              ("ks884xIntr(): Transmit UnderRun error!"NEWLINE));
    }

	if( wIntStatus & INT_TX_STOPPED)
	{
        phw->m_cnCounter[ portIndex ][ OID_COUNTER_XMIT_INT_STOP ]++;

        /* Disable the interrupt bit */
        HardwareDisableInterruptBit( phw, INT_TX_STOPPED );

        /* Clear the interrupt INT_TX_STOPPED status */
        HardwareAcknowledgeTxStop( phw );

	    DRV_PRINT(DRV_DEBUG_ERROR, 
              ("ks884xIntr(): Transmit Process Stopped!"NEWLINE));
	}


    /*
     * PHY Status
     */

    if ( ( wIntStatus & INT_PHY ) ) 
    {

        SwitchGetLinkStatus( phw );

        /* Acknowledge the interrupt. */
        HardwareAcknowledgeLink( phw );
    }

    /*
     * Receive the Packet Status
     */

    /*  Receive error */
    if ( ( wIntStatus & INT_RX_OVERRUN ) ) 
    {
        phw->m_cnCounter[ portIndex ][ OID_COUNTER_RCV_NO_BUFFER ]++;

        HardwareAcknowledgeOverrun( phw );

	    DRV_PRINT(DRV_DEBUG_RX, 
              ("ks884xIntr(): Receive OverRun error!"NEWLINE));
    }

	if( wIntStatus & INT_RX_ERROR)
	{
        phw->m_cnCounter[ portIndex ][ OID_COUNTER_RCV_INT_ERROR ]++;

        /* Clear the interrupt INT_RX_ERROR status */
        HardwareAcknowledgeErrorFrame( phw );

	    DRV_PRINT(DRV_DEBUG_ERROR, 
              ("ks884xIntr(): Receive Error Frame!"NEWLINE));
	}

	if( wIntStatus & INT_RX_STOPPED)
	{
        phw->m_cnCounter[ portIndex ][ OID_COUNTER_RCV_INT_STOP ]++;

        /* Disable the interrupt bit */
        HardwareDisableInterruptBit( phw, INT_RX_STOPPED );

        /* Clear the interrupt INT_RX_ERROR status */
        HardwareAcknowledgeRxStop( phw );

	    DRV_PRINT(DRV_DEBUG_ERROR, 
              ("ks884xIntr(): Receive Process Stopped!"NEWLINE));
	}

    /* Receive the completed packet. */
    if ( ( wIntStatus & INT_RX ) ) 
    {
        phw->m_cnCounter[ portIndex ][ OID_COUNTER_RCV_INT ]++;

#ifdef TEST_THROUGHPUT

        /* 
         * Driver performance test, drop the received packet, 
         * not pass to the upper layer protocol.
         */

        /* Disable the interrupt bit */
        HardwareDisableInterruptBit( phw, INT_RX );

        /* Acknowledge the RX interrupt. */
        HardwareAcknowledgeReceive( phw );

        /* isolate from OpenTCP stack */
        ksReceiveFrame();

        /* we already received completed packet, not need to process INT_RX_EARLY */
        if ( ( wIntStatus & INT_RX_EARLY ) ) 
        {
            /* Acknowledge the RX_EARLY interrupt. */
            HardwareAcknowledgeEarly( phw );
            wIntStatus &= ~(INT_RX_EARLY);
        }

        /* Enable the interrupt bit on the device-side */
        HardwareEnableInterruptBit ( phw, (USHORT)(INT_RX) );

#else  
        /* 
         * normal receive operation, pass received packet to 
         * the upper layer protocol.
         */
		rx_buffer_filled = 3;		/* signal that we want the buffer to get filled */
		NE2000ReceiveFrame();

#endif    

	}


    /* Receive the Early_Receive packet. */
    if ( ( wIntStatus & INT_RX_EARLY ) ) 
    {
        /*HardwareTurnOffEarlyInterrupt( phw );*/

        /* Acknowledge the RX_EARLY interrupt. */
        HardwareAcknowledgeEarly( phw );

        /* isolate from OpenTCP stack */
        ksReceiveEarlyFrame(); 

#if (1)
        /* Read the device interrupt status register again, incase we miss INT_RX */
        HardwareReadInterrupt( phw, &wIntStatus );

	    DRV_PRINT(DRV_DEBUG_INT, 
              ("ks884xIntr(): IntStatus=0x%04X"NEWLINE, wIntStatus));

#endif

	}

    /* Restore address within the TXQ or RXQ frame buffer. */
    HardwareRestoreRegs( phw );


#endif /* BRINGUP_BOARD */


}
#endif

#if (POLLED_ETHERNET)
#pragma INTERRUPT ks884xIntr
void ks884xIntr(void)
{
	// dummy function to satisfy the vector table entry
}
#endif

/*---------------------------------------------------------------------------*/

/*******************************************************************************
*
* ksReceiveEarlyFrame - Receive a Early Receive the Ethernet frame from NIC
*
*	This routine is to test ks884x Early Receive function. Read the 
*   Ethernet frame from ks884x packet memory by its Early Receive Threshold,
*   then read the rest Ethernet frame from ks884x packet memory.
*
* RETURNS: 
*
* NOMANUAL
*/
UINT8 ksReceiveEarlyFrame (void)
{
    int     packet_len = phw->m_wReceiveThreshold;
    int     port = 0, portIndex = 0;
    USHORT  rxCntl;
    USHORT  rxLength;
    UINT8   status=TRUE;
    int     bufOffset = phw->m_wReceiveThreshold;


	DRV_PRINT(DRV_DEBUG_RX, 
              ("ksReceiveEarlyFrame(): Start."NEWLINE ));

    /* check if hardware packet memory is is empty */
    if ( (HardwareReceiveMoreDataAvailable ( phw ) ) <= 0  )
        return;

    /* 
     * Read the Early Received data from hardware packet memory to receive buffer 
     * by Early Receive Threshold length
     */

    /* Move rx pointer to data address by read out 4-byte of received packet header from hardware packet memory */
    HardwareReceiveBuffer( phw, eth_pkt_rx_buf_temp, 4 );

    /* Read received packet from hardware packet memory */
    HardwareReceiveBuffer( phw, eth_pkt_rx_buf_temp, packet_len );

    /* dump received data to the console port for debug */
    if ( phw->fDebugDumpRx )
    {
         /* for debug use */
         DBG_PRINT( NEWLINE );
         DBG_PRINT("Received Early Receive Frame: "NEWLINE );
         if ( packet_len > 0 )
             PrintPacketData( (char *)eth_pkt_rx_buf_temp, packet_len, port, 2 );
    }

    /* 
     * Read the rest Ethernet frame from ks884x packet memory   
     */

    bufOffset = packet_len;

    /* Checking Received packet status */
    if ( HardwareReceiveStatus( phw, &rxCntl, &rxLength)  )
    {
        /*
         *  Receive Valid and Good Frame.
         */

        /* Get received packet length from hardware packet memory */
        packet_len = rxLength;
        packet_len -= bufOffset;

        /* update the received port */
        port = phw->m_bPortRX;
        portIndex = phw->m_bPortRX;
        if ( portIndex > 0 )
            portIndex -- ;


        /* 
         *  Packet must be checked with no errors, AND 
         *  the packet data size can't be bigger than our receive buffer. 
         */
        if ( (packet_len > 0)  && (packet_len <= (RX_BUF_SIZE - bufOffset)) )
        {
            /*  received Packet with no errors  */

            /* Read the received data from hardware packet memory to receive buffer */
            HardwareReceiveBuffer( phw, &eth_pkt_rx_buf_temp[bufOffset], packet_len );

            /* Update receive statistics. */
            phw->m_cnCounter[ portIndex ][ OID_COUNTER_DIRECTED_FRAMES_RCV ]++;
            phw->m_cnCounter[ portIndex ][ OID_COUNTER_DIRECTED_BYTES_RCV ] += bufOffset;
            phw->m_cnCounter[ portIndex ][ OID_COUNTER_DIRECTED_BYTES_RCV ] += packet_len;

            /* dump received data to the console port for debug */
            if ( phw->fDebugDumpRx )
            {
                /* for debug use */
                DBG_PRINT( NEWLINE );
                DBG_PRINT("Rx Cntl Word: 0x%04x, Byte Count: 0x%04x"NEWLINE, 
                          (rxCntl),
                          (rxLength) );
                if ( packet_len > 0 )
                {
                    PrintPacketData( (char *)&eth_pkt_rx_buf_temp[bufOffset], packet_len, port, 2 );
                }
            }

        } 
        else
        {
            status = FALSE;

            /*  received Packet length errors  */
	        DRV_PRINT(DRV_DEBUG_ERROR, 
                     ("ksReceiveEarlyFrame(): Receive packet length errors! packet_len=%d)"NEWLINE, 
                        packet_len ));

        }
    }
    else
    {
        /*  received Completed Packet with errors  */

        phw->m_cnCounter[ portIndex ][ OID_COUNTER_DIRECTED_BYTES_RCV ] -= packet_len;

        status = FALSE;

        DRV_PRINT(DRV_DEBUG_ERROR, 
                  ("ksReceiveEarlyFrame(): Receive packet with errors! ( rxCntl=0x%04x, rxLength=0x%04x)"NEWLINE, 
                    rxCntl, rxLength ));

    } /* if ( HardwareReceiveStatus( phw, &rxCntl, &rxLength)  ) */


    /* Free up that memory */
    HardwareReleaseReceive( phw );

    /* Acknowledge the RX interrupt. */
    HardwareAcknowledgeReceive( phw ); 

    /* 
     * To receive the rest of frame if they are more continuous incoming packets 
     */
    if ( status == TRUE )
        status = ksReceiveFrame();

    return(status);			/* Indicate we got packet or not */

}  /* ksReceiveEarlyFrame */

/*******************************************************************************
*
* ksReceiveFrame - Receive the Ethernet frame from NIC
*
*	This routine is to test ks884x throughput performance, just read the 
*   Ethernet frame from ks884x packet memory, and drop it, not pass to OpenTCP 
*   protocol stack.
*
* RETURNS: 
*
* NOMANUAL
*/

UINT8 ksReceiveFrame (void)
{
    int     packet_len = 0;
    int     port = 0, portIndex = 0;
    USHORT  rxCntl;
    USHORT  rxLength;
    UINT8   status=TRUE;
    USHORT  wMemLength;
    int     maxReadCount = 50;



	DRV_PRINT(DRV_DEBUG_RX, 
              ("ksReceiveFrame(): Start."NEWLINE ));

    /* while loop reading data from hardware packet memory, until packet memory is empty */
    while  ( 
             ( (wMemLength = HardwareReceiveMoreDataAvailable ( phw )) > 0 ) 
             /* ( && (maxReadCount--) > 0  )  */
             /* && ( status ) */
           )
    {
	    DRV_PRINT(DRV_DEBUG_RX, 
                  ("ksReceiveFrame(): wMemLength=%d)"NEWLINE, 
                    wMemLength ));

        /* Checking Received packet status */
        if ( HardwareReceiveStatus( phw, &rxCntl, &rxLength)  )
        {
            /*
            *  Receive Valid and Good Frame.
            */

            /* Get received packet length from hardware packet memory */
            packet_len = HardwareReceiveLength( phw );

            /* update the received port */
            port = phw->m_bPortRX;
            portIndex = phw->m_bPortRX;
            if ( portIndex > 0 )
               portIndex -- ;

            /* 
             *  Packet must be checked with no errors, AND 
             *  the packet data size can't be bigger than our receive buffer. 
             */
            if ( (packet_len > 0) && (packet_len <= RX_BUF_SIZE) )
            {
                /*  received Packet with no errors  */

                /* Read the received data from hardware packet memory to receive buffer */
                HardwareReceiveBuffer( phw, &eth_pkt_rx_buf_temp[0], packet_len );

                /* Update receive statistics. */
                phw->m_cnCounter[ portIndex ][ OID_COUNTER_DIRECTED_FRAMES_RCV ]++;
                phw->m_cnCounter[ portIndex ][ OID_COUNTER_DIRECTED_BYTES_RCV ] += packet_len;

                /* dump received data to the console port for debug */
                if ( phw->fDebugDumpRx )
                {
                    /* for debug use */
                    DBG_PRINT( NEWLINE );
                    DBG_PRINT("Rx Cntl Word: 0x%04x, Byte Count: 0x%04x"NEWLINE, 
                        (rxCntl),
                        (rxLength) );
                    if ( packet_len > 0 )
                    {
                         PrintPacketData( (char *)eth_pkt_rx_buf_temp, packet_len, port, 2 );
                    }
                }
            } 
            else
            {
                status = FALSE;

               /*  received Packet length errors  */
	           DRV_PRINT(DRV_DEBUG_ERROR, 
                     ("ksReceiveFrame(): Receive packet length errors! packet_len=%d)"NEWLINE, 
                        packet_len ));

            }
        }
        else
        {
            status = FALSE;

            /*  received Packet with errors  */
	        DRV_PRINT(DRV_DEBUG_ERROR, 
                  ("ksReceiveFrame(): Receive packet with errors! ( rxCntl=0x%04x, rxLength=0x%04x)"NEWLINE, 
                    rxCntl, rxLength ));

        } /* if ( HardwareReceiveStatus( phw, &rxCntl, &rxLength)  ) */


        /* Free up that memory */
        HardwareReleaseReceive( phw ); 

    } /* while  ( HardwareReceiveMoreDataAvailable ( phw ) > 0 ) */


	DRV_PRINT(DRV_DEBUG_RX, 
                  ("ksReceiveFrame(): wMemLength=%d, status=%d)"NEWLINE, 
                    wMemLength, status ));

	DRV_PRINT(DRV_DEBUG_RX, 
              ("ksReceiveFrame(): Stop."NEWLINE ));

    return(status);			/* Indicate we got packet or not */

}

/*******************************************************************************
*
* ksSendFrame - Send the Ethernet frame to NIC
*
*	Send the Ethernet frame to NIC directly without through NETWORK_COMPLETE_SEND() macro.
*   to test throughput performace.
*
* RETURNS: void.
*
* NOMANUAL
*/
UINT8 ksSendFrame 
( 
    UINT8 * pData,
    UINT16  dataLength,
    ULONG   port
)
{
 	/* After filling Tx buffer, call this function to send it */
 	/* Input: 	dataLength: packet length  					  */

    ULONG      txCntlAndLength=0;
    USHORT     txCntl;
    USHORT     txLength;
    ULONG      portIndex=0;


    
    /* Set trasnmit destination port */
    if (port > 3)
	    return (FALSE);

    phw->m_bPortTX = port;
    portIndex = phw->m_bPortTX;


    /* Checking ks8842 TXQ memory available for this packet */
    if ( ! HardwareAllocPacket( phw, dataLength ) ) 
    {
	    DRV_PRINT(DRV_DEBUG_TX, 
              ("ksSendFrame(): device TXQ memory not available!"NEWLINE));
	    return (FALSE);
    }


    /* prepares the transmit 'write' to 'TXQ', write into beginning of ks8842 
       packet memory with 4-byte control information (packet length, and destination port) */


    txCntlAndLength = HardwareSetTransmitLength( phw, dataLength );

	/* Now write the data out to the Ethernet chip */
    /* Continue write packet data (in WORD) into ks8842 packet memory */
    HW_WRITE_BUFFER( phw, REG_DATA_OFFSET, pData, dataLength );

    /* Issue TXQ Command (Enqueue Tx frame from TX buffer into TXQ for transmit) */
    if ( HardwareSendPacket( phw ) ) 
    {
        /* Update driver transmit counters */
        if ( portIndex == 3 )
        {
            while ( portIndex & 3)
            {
               portIndex >>= 1;
               phw->m_cnCounter[ portIndex ][ OID_COUNTER_DIRECTED_FRAMES_XMIT ]++;
               phw->m_cnCounter[ portIndex ][ OID_COUNTER_DIRECTED_BYTES_XMIT ] += dataLength;
            }
        }
        else
        {
               if ( portIndex != 0 )
                   portIndex --;
               phw->m_cnCounter[ portIndex ][ OID_COUNTER_DIRECTED_FRAMES_XMIT ]++;
               phw->m_cnCounter[ portIndex ][ OID_COUNTER_DIRECTED_BYTES_XMIT ] += dataLength;
        }
    }

    /* dump transmit data to the console port for debug */
    if ( phw->fDebugDumpTx )
    {
        /* for debug use */
        DBG_PRINT( NEWLINE );
        txCntl   = (txCntlAndLength & 0x0000ffff);
        txLength = (txCntlAndLength >> 16) & 0x0000ffff;
        DBG_PRINT("Tx Cntl Word: 0x%04x, Byte Count: 0x%04x"NEWLINE, 
                    (txCntl),
                    (txLength) );
        PrintPacketData( (char *)pData, dataLength, port, 1 );
    }

	return (TRUE);
}


/*******************************************************************************
*
* ksEnetAddrSet - Set ks884x Ethernet (IEEE station) address
*
* This routine provides an interface for accessing an ks884x device
* Ethernet (IEEE station address) address in the device.
* The 6-byte IEEE station address, specified by the <enetAdrs> parameter, 
* will be wrote to ks884x device MAC address registers .
*
*
* RETURNS: OK, or ERROR if could not be obtained.
*
* NOMANUAL
*/
static UINT8 ksEnetAddrSet
(
    char *     enetAdrs    /* specifies storage for station address */
)
{
    UCHAR         i;


	/* 
     * Set MAC address to the Station registers 
     * ADDRESS 0 corresponds to the first address byte on the cable 
     * OpenTCP defines the localHW[6] array as [5]:[4]:[3]:[2]:[1]:[0] 
     */

    phw->m_bMacOverrideAddr = TRUE;
    for (i=0; i<MAC_ADDRESS_LENGTH; i++)
         phw->m_bOverrideAddress[MAC_ADDRESS_LENGTH-1-i] = enetAdrs[i];
    HardwareSetAddress( phw );

	/* Get MAC address from the Station registers */
    HardwareReadAddress( phw );

 	if (phw->m_bPermanentAddress[0] != enetAdrs[5] ||
        phw->m_bPermanentAddress[1] != enetAdrs[4] ||
		phw->m_bPermanentAddress[2] != enetAdrs[3] || 
        phw->m_bPermanentAddress[3] != enetAdrs[2] ||
		phw->m_bPermanentAddress[4] != enetAdrs[1] || 
        phw->m_bPermanentAddress[5] != enetAdrs[0]) 
        return (FALSE);

    return (TRUE);
}


/**
 *  Convert device ID to device name. 
 *
 */ 
static void getDeviceName 
(
    USHORT   ChipID
)
{
    if (ChipID == CHIP_IP_41_ISA)
        fKS884x = 1;
    else if (ChipID == CHIP_IP_42_ISA)
        fKS884x = 2;
    else if (ChipID == CHIP_IP_61_ISA)
        fKS884x = 3;
    else if (ChipID == CHIP_IP_62_ISA)
        fKS884x = 4;
    else
        fKS884x = 0;  /* unknown device */
}

/**
 *  Display KS884x BSP info on UART 
 *
 */ 
static void showKsBspInfo 
(
)
{
    unsigned char chipBaseAddr[4];
    USHORT   ChipID;
    UCHAR    DevRevisionID;



#ifndef KS_QC_TEST

    HardwareReadChipID ( phw, &ChipID, &DevRevisionID );
    getDeviceName ( ChipID );

    DisplayString(LCD_LINE2, OpenTCP_EthDriver_ver);

    /* Display Device Driver info on UART */
    DBG_PRINT( NEWLINE );
	DBG_PRINT("Micrel - Ethernet Group."NEWLINE);	   

#ifndef TEST_THROUGHPUT 

#ifdef KS_ISA_8BIT_BUS
    DBG_PRINT("%s 8-bit BUS Ethernet Driver demo."NEWLINE, deviceName[fKS884x] );
#else
    DBG_PRINT("%s 16-bit BUS Ethernet Driver demo."NEWLINE, deviceName[fKS884x] ); 
#endif

    /* OpenTCP Ethernet Driver Version */
    DBG_PRINT("OpenTCP Ethernet Driver Version: %s" NEWLINE, OpenTCP_EthDriver_ver);	 

#else  /* #ifndef TEST_THROUGHPUT */

#ifdef KS_ISA_8BIT_BUS
    DBG_PRINT("%s 8-bit BUS Test Throughput."NEWLINE, deviceName[fKS884x] );
#else
    DBG_PRINT("%s 16-bit BUS Test Throughput."NEWLINE, deviceName[fKS884x] );
#endif

    /* OpenTCP Ethernet Driver Version */
    DBG_PRINT("OpenTCP Ethernet Driver Version: %s" NEWLINE, Througput_EthDriver_ver);	 

#endif /* #ifndef TEST_THROUGHPUT */

    /* KS884X Driver Version */
    DBG_PRINT("%s Driver Version: " KS884X_VERSION, deviceName[fKS884x]); 
    DBG_PRINT( NEWLINE );

    DBG_PRINT("%s Chip ID       : %04x", deviceName[fKS884x], ChipID);     /* KS884X Chip ID */
    DBG_PRINT( NEWLINE );
    DBG_PRINT("%s Device Revision: %d", deviceName[fKS884x], DevRevisionID); /* KS884X Device Revision */
    DBG_PRINT( NEWLINE );


#if defined (EARLY_TRANSMIT) && defined (EARLY_RECEIVE)
    DBG_PRINT("Early Trasmit\\Receive Function is included. "NEWLINE); 
#endif

    memcpy ((char *)&chipBaseAddr[0],  (char *)&phw->m_ulVIoAddr, 4 );
    DBG_PRINT( "%s Base Address: 0x%02x%02x%02x%02x"NEWLINE, 
                deviceName[fKS884x],
                chipBaseAddr[3],
                chipBaseAddr[2],
                chipBaseAddr[1],
                chipBaseAddr[0] );

    DBG_PRINT("MAC Address: ");	   
    PrintMacAddress( phw->m_bPermanentAddress );
    DBG_PRINT( NEWLINE );
	
    DBG_PRINT("IP  Address: ");	   
    PrintIpAddress( localmachine.localip );
    DBG_PRINT( NEWLINE );

#else  

   /* For QC FAB testing */
    DBG_PRINT( NEWLINE );
    DBG_PRINT( "^[ok : %s]$"NEWLINE, QC_EthDriver_ver );

    DisplayString(LCD_LINE2, QC_EthDriver_ver);

#endif  /* #ifndef KS_QC_TEST */

}

/**
 *  Display KS884x Link Status on UART 
 *
 */ 
static void showKsLinkStatus 
(
)
{
    char  lcdStr[9];
#ifndef KS_QC_TEST
    PPORT_INFO pInfo;
    UCHAR      bPort;


    /* Display OpenTCP Ethernet Driver Version on LCD */

    DisplayString(LCD_LINE1, "Version ");

#ifndef TEST_THROUGHPUT 
    DisplayString(LCD_LINE2, OpenTCP_EthDriver_ver);
#else
    DisplayString(LCD_LINE2, Througput_EthDriver_ver);
#endif
    DelayMillisec(2000);			/* 2 sec delay to read message */		

    /* Display Ethernet Link status on UART */
#ifdef DEF_KS8842
    for ( bPort = 0; bPort < SWITCH_PORT_NUM; bPort++ )
#else
    bPort = 0;
#endif
    {
        sprintf( lcdStr, "Port %d  ", (bPort+1) );
        DisplayString(LCD_LINE1,lcdStr );
        DisplayString(LCD_LINE2,"Eth Link");
        DelayMillisec(2000);			/* 1 sec delay to read message */		

        pInfo = &phw->m_PortInfo[ bPort ];

        DBG_PRINT("Port %d Link Status:  ", (bPort+1) );	   

        if ( pInfo->ulHardwareState == MediaStateConnected )
        {
	        eth_info.linked = 1;

	        /* Speed */
            if (pInfo->ulSpeed == 1000000)
            {
		        eth_info.speed_100 = 1;
	            DBG_PRINT("100 Mbps, ");	   
		        DisplayString(LCD_LINE1,"100 Mbps");
            }
            else if (pInfo->ulSpeed == 100000)
            {
	            DBG_PRINT("10  Mbps, ");	   
 		        DisplayString(LCD_LINE1,"10 Mbps ");	
            }
            else 
            {
	            DBG_PRINT("??? Mbps, ");	   
		        DisplayString(LCD_LINE1,"??? Mbps");	
            }

	        /* Duplex */
            if ( pInfo->bDuplex == 2)
            {
		        eth_info.full_duplex = 1;
	            DBG_PRINT("Full DPX");	   
		        DisplayString(LCD_LINE2,"Full DPX");
            }
	        else if ( pInfo->bDuplex == 1)
            {
	            DBG_PRINT("Half DPX");	   
		        DisplayString(LCD_LINE2,"Half DPX");	
            }	
            else 
            {
	            DBG_PRINT("??? DPX");	   
		        DisplayString(LCD_LINE2,"??? DPX ");	
            }
        }
        else
        {
	        DBG_PRINT("Down ");	   
	        DisplayString(LCD_LINE1,"Eth Link");
	        DisplayString(LCD_LINE2,"No Conn ");
        }
        DelayMillisec(2000);			/* 2 sec delay to read message */		
        DBG_PRINT( NEWLINE );

    } /* for ( bPort = 0; bPort < SWITCH_PORT_NUM; bPort++ ) */

    DBG_PRINT( NEWLINE );

#endif
}


static BOOLEAN QCCheckChipID
(
    PHARDWARE phw 
)
{
    USHORT wData;


    /*
     * Set Bus Speed to 125MHz
     */

    HardwareSelectBank( phw, REG_BUS_CTRL_BANK );
    HW_WRITE_WORD ( phw, REG_BUS_CTRL_OFFSET, BUS_SPEED_125_MHZ );

    /*
     * Check ks884x Chip ID
     */

    HardwareSelectBank( phw, REG_SWITCH_CTRL_BANK );
    HW_READ_WORD( phw, REG_CHIP_ID_OFFSET, &wData );

    if  ( (( wData & SWITCH_CHIP_ID_MASK ) != REG_CHIP_ID_41 ) &&
          (( wData & SWITCH_CHIP_ID_MASK ) != REG_CHIP_ID_42 ) )
    {
        return (FALSE);
    }

    return( TRUE );

}  /* HardwareInitialize */

