/*********************************************************
	Property of Renesas Technology America, Inc.
*********************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

int main(int argc, char *argv[]) 
{
	char filename[30];

	int total_tags = 0;
	int tag_pos[20];
	int search_for_tags = 0;

	char text[30];

	FILE *fin, *fout_c, *fout_h;
	
	int i, total_chars;

	if (argc < 3) 
	{
		printf("\nUsage:\n");
		printf("File2C.exe [Infile] [ram|flash]\n");
		printf("ram: Array will reside in RAM\n");
		printf("flash: Array will reside in Flash (const)\n");
		printf("Renesas Technology America, Inc.\n");
		exit(EXIT_SUCCESS);
	}

	if( (argv[2][0] != 'r') && (argv[2][0] != 'f') )
	{
		perror("RAM or Flash options must be 'ram' or 'flash'");
		exit(EXIT_FAILURE);
	}


	fin = fopen(argv[1], "rb");
	if (!fin)
	{
		perror(argv[1]);
		exit(EXIT_FAILURE);
	}

	/* check if the file is a HTML file */
	strcpy(text,argv[1]);
	_strupr(text);		// make uppercase
	i = strlen(text);
	if( !strcmp(text + i - 3,"HTM"))
	{
		search_for_tags = 1;

	}
	if( !strcmp(text + i - 4,"HTML"))
	{
		search_for_tags = 1;

	}
	
	/* Create the file names */
	strcpy(filename,argv[1]);
	i = 0;
	while(filename[++i] != '.');	// look for '.'
	filename[i] = '_';				// replace '.' with '_'
	while(filename[++i] != 0);	// look for NULL

	filename[i++] = '.';			// add .c
	filename[i++] = 'c';
	filename[i] = 0;				// move NULL

	fout_c = fopen(filename, "wt");
	if (!fout_c)
	{
		fclose(fin);
		perror("Can't open file for output");
		exit(EXIT_FAILURE);
	}

	filename[i - 1] = 'h';
	fout_h = fopen(filename, "wt");
	if (!fout_h)
	{
		fclose(fin);
		fclose(fout_c);
		perror("Can't open file for output");
		exit(EXIT_FAILURE);
	}
	
	fprintf(fout_c,"/* This file was created by the File2C utility. */\n");
	fprintf(fout_c,"/* Renesas Technology America, Inc. */\n\n");
//	fprintf(fout_c, "#define %s_C_INC\t1\n", ufname);
//	fprintf(fout_c, "#include \"%s.h\"\n", fname);

	/************ write out the array int .c file  **********/
	filename[i - 2] = 0;		// remove file extension from string to make array name

	if( argv[2][0] == 'r' )
		fprintf(fout_c, "\nunsigned char %s[]=\n{\n", filename);
	else
		fprintf(fout_c, "\nconst unsigned char %s[]=\n{\n", filename);

	total_chars = 0;
	while ((i=fgetc(fin)) != EOF) 
	{
		/* Search for tag symbols */
		if(search_for_tags && (i == '`'))
		{
			// don't copy tag characters, but record where in the file it occured.
			// also keep track of the number of bytes between the tages.
			tag_pos[total_tags] = total_chars;
			total_tags++;
			continue;			// don't count this character
		}

		fprintf(fout_c, " 0x%02x,",i);
		total_chars++;

		if(!(total_chars % 10) )
			fprintf(fout_c, "\n");	// newline ever 10 chars

	}

	fprintf(fout_c, "\n};\n");

    /* WRITE OUT THE HEADER FILE  */
	fprintf(fout_h,"/* This file was created by the File2C utility. */\n");
	fprintf(fout_h,"/* Renesas Technology America, Inc. */\n\n");
//	fprintf(fout_h, "#ifndef %s_H_INC\n", ufname);
//	fprintf(fout_h, "#define %s_H_INC\t1\n", ufname);
//	fprintf(fout_h, "\n#include \"mn_defs.h\"\n");

	fprintf(fout_h, "\n#define %s_SIZE %ld\n", filename, total_chars);
//	fprintf(fout_h, "\n#ifndef %s_C_INC\n", ufname);

	/* print out tags */
	fprintf(fout_h, "\n/* File Tag defines */\n");
	for(i=0;i<total_tags;i++)
	{
		fprintf(fout_h, "#define %s_TAG_%1d %1d\n",filename,i+1,tag_pos[i]);
	}

	if( argv[2][0] == 'r' )
		fprintf(fout_h, "\nextern unsigned char %s[];\n", filename);
	else
		fprintf(fout_h, "\nextern const unsigned char %s[];\n", filename);

	//	fprintf(outh, "#endif\n"); 
	//fprintf(outh, "\n#endif\t\t/* %s_H_INC */\n", ufname);

	/* wrap it up */
	fclose(fin);
	fclose(fout_c);
	fclose(fout_h);

	return (0);
}

