/* ks8842End_shBus.h - END Ethernet driver header for Micrel KS884X 32bit Generic Bus Ethernet driver */

/* Copyright 1998-2002 Wind River Systems, Inc. */
/* 
 *	Copyright (c) 2004-2005, Micrel Semiconductor
 *
 *	This END Ethernet driver is for Micrel's KS8841 or KS8842 32bit Generic Bus interface.
 */

/*
modification history
--------------------
*/
#ifdef KS_ISA_BUS

#ifndef __INCKS8842END_SHBUS_H
#define __INCKS8842END_SHBUS_H

#ifdef __cplusplus
extern "C" {
#endif

#include "end.h"
#include "cacheLib.h"
#include "netinet/if_ether.h"
#include "wdLib.h"
#include "hardware.h"

/* define the various levels of debugging if the DRV_DEBUG is defined */    

#ifdef DRV_DEBUG
#    include "logLib.h"
#endif /* DRV_DEBUG */


extern void hw_read_byte  ( PHARDWARE, UCHAR, UINT8 * );
extern void hw_write_byte ( PHARDWARE, UCHAR, UINT8   );
extern void hw_read_word  ( PHARDWARE, UCHAR, UINT16 *);
extern void hw_write_word ( PHARDWARE, UCHAR, UINT16  );

/*
 * KS884X_MAX_IF is 1 if it is KS8841, 2 if it is KS8842.
 */
#define KS884X_MAX_IF            1			/* Max ks884X END device */


/* Definitions for the drvCtrl specific flags field */

#define KS_PROMISCUOUS_FLAG     0x001   /* set the promiscuous mode */
#define KS_MEM_ALLOC_FLAG       0x002   /* allocating memory flag */
#define KS_POLLING              0x020   /* polling flag */
#define KS_MULTICAST_FLAG       0x040   /* set the receive all multicast frames */
#define KS_MODE_MEM_IO_MAP      0x100   /* device registers memory mapped */
#define KS_MODE_DWIO            0x200   /* device in 32 bit mode */
#define KS_MODE_NOSWAP_MASTER	0x400   /* don't swap PCI master I/O ops */

/* standard ethernet header */
#define ENET_HEADER_SIZE                14
#define MINIMUM_ETHERNET_PACKET_SIZE	60
#define ETH_CRC_LENGTH					4
#define ETH_LENGTH_OF_ADDRESS			6

/* Definitions for PHY status */
#define SPEED_UNKNOWN					0
#define SPEED_10						10
#define SPEED_100						100
#define FULL_DUPLEX						1		/* default for full duplex */
#define HALF_DUPLEX						2


/* Configuration items */

#define TXBUF_DEFAULT		32            /* default number of receive buffer */
#define RXBUF_DEFAULT		32            /* default number of transmit buffer */

#define KS_BUFSIZ               (ETHERMTU + SIZEOF_ETHERHEADER + 6) /* Ethernet buffer size */
#define LN_SPEED                (10000000)
#define KS_DEV_NAME             "ks"      /* name of the device */
#define KS_DEV_NAME_LEN         (3)       /* length of the name string */
#define KS_LITTLE_ENDIAN        (0)       /* little endian PCI bus      */
#define KS_BIG_ENDIAN           (1)       /* big endian PCI bus         */
#define KS_ACTIVE_LOW           (0)       /* active-low interrupt line  */
#define KS_ACTIVE_HI            (1)       /* active-high interrupt line */


/* Configuration items */


/* mBlks, clBlks, and cluster buffer */
typedef struct ks8842_buffer 
{
    M_BLK_ID 	pMblk;
} KS_MBLK;


/* ks884x DMA control structure */

typedef struct _DMA_INFO 
{
	INT32	nBaseAddr;			    /* base address */
	UINT32  nResetCount;			/* DMA reset counter */

	/* mac */
    UINT8   stMacStation[MAC_ADDRESS_LENGTH];

	/* Tx related */
	UINT8	bTxStarted;			    /* Tx DMA started or stopped! */
	UINT8	bTxFlowCtrl;		    /* flow control for Tx DMA */
	INT32	nTxBufTotal;		    /* total number for Tx buffer */

    KS_MBLK *pTxMblk;               /* House keeping of Tx Mblk */

	/* Rx related */
	UINT8	bRxStarted;			    /* Rx DMA started or stopped! */
	UINT8	bRxFlowCtrl;		    /* flow control for Rx DMA */
    UINT8   bRxPromiscuous;		    /* enable rx promiscuous mode */
    UINT8   bRxMulticast;		    /* enable rx all multicast frames */
    UINT32  uRxBufferLen;		    /* rx buffer length */
	INT32	nRxBufTotal;		    /* total number fo Rx buffer */

    KS_MBLK *pRxMblk;               /* House keeping of Rx Mblk */

	UINT8   bPowerDownReset;		/* perform powerdown reset instead of soft reset */

} DMA_INFO, *PDMA_INFO;


/* k884x switch port control structure */

typedef struct _PORT_INFO 
{
	/* PHY related */
	UINT8	bAutoNegoInProgress;    /* if set, means that auto nego is in progress!!! */
	UINT8	bLinkActive;		    /* flag indicates whether link is active or not */
    UINT8   bLinkChanged;		    /* link changed indicator */
	UINT16  myLinkType;			    /* Link type - auto-negotiation or manually */
    UINT16  myLinkSpeed;		    /* link speed */
    UINT8   myLinkDuplex;		    /* link duplex (full/half) */

    UINT16  hisLinkSpeed;		    /* link speed based on partner's settings */
    UINT8   hisLinkDuplex;	        /* HD/FD detected based on partner's settings */

	INT32	nLinkChangeCount;		/* trace link change count */

	/* PORT related */
	UINT8	bPortsFlowCtrl;		    /* flow control */
    UINT16  usTag;				    /* tag value for the port (ingress) */
	UINT8   byStormProtection;	    /* broadcast storm protection */
	UINT8   bySTPenable;		    /* spanning tree enable\disable control */
	UINT8   bySTPstate;	            /* STP states (disable\block\listern\learn\forward) */
	UINT8   byIngressPriority;	    /* ingress priority */
	UINT8   byIngressPriorityTOS;	/* TOS based ingress priority */
	UINT8   byIngressPriority802_1P;/* 802.1p based ingress priority */
	UINT8   byEgressPriority;		/* egress priority */

} PORT_KS_INFO, *PPORT_KS_INFO;



/* The definition of the driver control structure */

typedef struct drv_ctrl
{
    END_OBJ           endObj;            /* The class we inherit from */
    int               unit;              /* unit number of the device */

    int               inum;              /* interrupt number */
    int               ilevel;            /* interrupt level */

    volatile char *   pShMem;            /* real ptr to shared memory */
    volatile char *   memBase;           /* PCnet memory pool base */
    volatile char *   memAdrs;           /* PCnet memory pool base */

    UINT32            devAdrs;           /* device structure address */
    UINT16            boardBusEndianMode;/* board bus endian mode board specific */
    int               memSize;           /* PCnet memory pool size */
    int               memWidth;          /* width of data port */
    int               offset;            /* offset of data in the buffer */
    volatile UINT32   flags;             /* Our local flags */
    UINT8             enetAddr[6];       /* ethernet address */
    CACHE_FUNCS       cacheFuncs;        /* cache function pointers */

    volatile BOOL     txCleaning;        /* transmit descriptor cleaning */
    volatile BOOL     txBlocked;         /* transmit flow control */
    volatile BOOL	  rxHandling;		 /* receive frames handling */
    volatile BOOL     errorHandling;     /* task level error handling */
    END_ERR           lastError;         /* Last error passed to muxError */

    CL_POOL_ID        pClPoolId;         /* cluster pool Id */
    M_CL_CONFIG       mClCfg;            /* mBlk & cluster config structure */
    CL_DESC           clDesc;            /* cluster descriptor table */

	PDMA_INFO	      pDMAInfo;		     /* ks884x DMA information */
	PORT_KS_INFO	  port[TOTAL_PORT_NUM]; /* ks884x port related (switch port only) */
    PHARDWARE         pHW;               /* The pointer to the ks884x HW structure. */

	WDOG_ID	          watchdog;			 /* watch dog id */
	int	              nWatchdogDelay;	 /* watch dog delay period */

	UINT32            fPollSendFromCLI;	 /* flag to indicate call ks884xEndPollSend from CLI hwbuftx command */
} DRV_CTRL;

IMPORT DRV_CTRL *gpDrvCtrl[];
#define	DI		(pDrvCtrl->pDMAInfo)	 /* SWITCH DMA Inforamtion */



#ifdef __cplusplus
}
#endif

#endif /* __INCKS8842END_SHBUS_H */

#endif /* #ifdef KS_ISA_BUS */
