/* sysALib.s - Hitachi ULSI MS7751SE system-dependent assembly routines */

/* Copyright 1984-2001 Wind River Systems, Inc. */

        .data
        .globl  _copyright_wind_river
        .long   _copyright_wind_river

/*
modification history
--------------------
01b,15apr02,zl   corrected comments in intPrioTable.
01a,23apr01,zl   written.
*/

/*
DESCRIPTION
This module contains system-dependent routines written in assembly language.

This module must be the first specified in the \f3ld\f1 command used to
build the system.  The sysInit() routine is the system start-up code.
*/

#define _ASMLANGUAGE
#include "vxWorks.h"
#include "sysLib.h"
#include "asm.h"
#include "config.h"

	/* internals */

	.global	_sysInit		/* start of system code */
	.global	_sysOutByte
	.global	_sysOutWord
	.global	_sysOutLong
	.global	_sysInByte
	.global	_sysInWord
	.global	_sysInLong
	.global	_intPrioTable
	.global	_intPrioTableSize

	/* externals */

	.extern	_usrInit		/* system initialization routine */
	.extern	_intLockTaskSR


/*******************************************************************************
*
* sysInit - start after boot
*
* This is the system start-up entry point for VxWorks in RAM, the
* first code executed after booting.  It disables interrupts, sets up
* the stack, and jumps to the C routine usrInit() in usrConfig.c.
*
* The initial stack is set to grow down from the address of sysInit().  This
* stack is used only by usrInit() and is never used again.  Memory for the
* stack must be accounted for when determining the system load address.
*
* NOTE: This routine should not be called by the user.
*
* RETURNS: N/A

* sysInit ()              /@ THIS IS NOT A CALLABLE ROUTINE @/

*/
	.text
	.align 	5
	.type	_sysInit,@function

_sysInit:
	mov.l	IntLockTaskSR,r1
	mov.l	@r1,r0
	ldc	r0,sr			/* disable interrupts               */

	mov.l	SysInit,sp		/* stack grows down from code start */

	mov.l	UsrInit,r0
	mov.l	StartType,r4		/* set start type arg for usrInit() */
	xor	r1,r1			/* call usrInit(int startType), and */
	jsr	@r0;			/* never returns - starts up kernel */
	ldc	r1,gbr			/* clean gbr for nicer ti() output  */

		.align	2
IntLockTaskSR:	.long	_intLockTaskSR
SysInit:	.long	_sysInit
UsrInit:	.long	_usrInit		/* bootConfig.c, usrConfig.c */
StartType:	.long	BOOT_WARM_AUTOBOOT	/* see sysLib.h              */

#ifndef IO_PORTABLE

/*******************************************************************************
*
* sysOutByte - write a byte to the passed address
*
* SEE ALSO:
*/
 
	.text
	.align 	5
	.type	_sysOutByte,@function
_sysOutByte:
	rts        
	mov.b	r5,@r4

/*******************************************************************************
*
* sysOutWord - write a word to the passed address
*
* SEE ALSO:
*/
 
	.type	_sysOutWord,@function
_sysOutWord:
	rts        
	mov.w	r5,@r4
  
/*******************************************************************************
*
* sysOutLong - write a long to the passed address
*
* SEE ALSO:
*/
 
	.type	_sysOutLong,@function
_sysOutLong:
	rts        
	mov.l	r5,@r4
 
/*******************************************************************************
*
* sysInByte - read a byte from the passed address
*
* SEE ALSO:
*/

	.type	_sysInByte,@function
_sysInByte:
	mov.b	@r4,r0
	rts
	extu.b	r0,r0


/*******************************************************************************
*
* sysInWord - read a word from the passed address
*
* SEE ALSO:
*/

	.type	_sysInWord,@function
_sysInWord:
	mov.w	@r4,r0
	rts
	extu.w	r0,r0

/*******************************************************************************
*
* sysInLong - read a long from the passed address
*
* SEE ALSO:
*/
 
	.type	_sysInLong,@function
_sysInLong:
	rts
	mov.l	@r4,r0

#endif /* IO_PORTABLE */


/*******************************************************************************
*
* intPrioTable - interrupt priority table
*
* This table defines the interrupt mask levels for all interrupt handlers and
* non-TLB mishit exception handlers.  These entries are lined up in the order
* of EXPEVT/INTEVT register value, and each entry needs to be written in a form
* of status register value.  When an interrupt is granted by CPU, VxWorks
* updates status register with a value in this table.  If the corresponding
* table entry is NULL, status register is not updated by VxWorks.
*
* The TLB mishit exception handler normally runs without unblocking interrupts/
* exceptions.  If it fails to find the new page table entry, it dispatches to
* the regular exception handler then a new status register value is applied
* from this table.
*
* The first two entries in this table are originally assigned for `power on
* reset' and `manual reset' exceptions.  However, these entries are not used
* for their original exceptions since those exceptions have a fixed vectoring
* address at 0xa0000000.  Therefore VxWorks uses these two entries for special
* purposes.  The first entry is used for software breakpoint, and the second
* entry is used for integer zero divide detection.
*/
	.align	5
	.type	_intPrioTable,@object

_intPrioTable:			/*EXPEVT*/	/*INTEVT*/
	.long	NULL		/* 0x000: ENTRY NOT USED (power on reset)    */
	.long	NULL		/* 0x020: ENTRY NOT USED (manual reset)      */
	.long	0x400000f0	/* 0x040: Instruction TLB miss/invalid       */
				/*       & Data TLB miss (read)              */
	.long	0x400000f0	/* 0x060: Data TLB miss/invalid (write)      */
	.long	0x400000f0	/* 0x080: initial page write                 */
	.long	0x400000f0	/* 0x0a0: Data TLB protection (read)         */
	.long	0x400000f0	/* 0x0c0: Data TLB protection (write)        */
	.long	0x400000f0	/* 0x0e0: address error (read)               */
	.long	0x400000f0	/* 0x100: address error (write)              */
	.long	0x400000f0	/* 0x120: FPU exception                      */
	.long	0x400000f0	/* 0x140: Instruction TLB multiple-hit       */
	 			/*       & Data TLB multiple-hit             */
	.long	0x400000f0	/* 0x160: unconditional trap                 */
	.long	0x400000f0	/* 0x180: illegal instruction                */
	.long	0x400000f0	/* 0x1a0: illegal slot instruction           */
	.long	NULL				/* 0x1c0: NMI                */
	.long	0x400000f0	/* 0x1e0: user breakpoint trap               */
	.long	0x400000f0			/* 0x200: IRL15              */
	.long	0x400000e0			/* 0x220: IRL14              */
	.long	0x400000d0			/* 0x240: IRL13              */
	.long	0x400000c0			/* 0x260: IRL12              */
	.long	0x400000b0			/* 0x280: IRL11              */
	.long	0x400000a0			/* 0x2a0: IRL10              */
	.long	0x40000090			/* 0x2c0: IRL9               */
	.long	0x40000080			/* 0x2e0: IRL8               */
	.long	0x40000070			/* 0x300: IRL7               */
	.long	0x40000060			/* 0x320: IRL6               */
	.long	0x40000050			/* 0x340: IRL5               */
	.long	0x40000040			/* 0x360: IRL4               */
	.long	0x40000030			/* 0x380: IRL3               */
	.long	0x40000020			/* 0x3a0: IRL2               */
	.long	0x40000010			/* 0x3c0: IRL1               */
	.long	0x400000f0			/* 0x3e0: (reserved)         */
	.long	0x40000000 |(INT_LVL_SYSCLK <<4)/* 0x400: TMU0_UNDERFLOW     */
	.long	0x40000000 |(INT_LVL_AUXCLK <<4)/* 0x420: TMU1_UNDERFLOW     */
	.long	0x40000000 |(INT_LVL_TSTAMP <<4)/* 0x440: TMU2_UNDERFLOW     */
	.long	0x400000f0			/* 0x460: TMU2_INPUT_CAPTURE */
	.long	0x400000f0			/* 0x480: RTC_ALARM          */
	.long	0x400000f0			/* 0x4a0: RTC_PERIODIC       */
	.long	0x400000f0			/* 0x4c0: RTC_CARRY          */
	.long	0x40000000 | (INT_LVL_SCI <<4)	/* 0x4e0: SCI_RX_ERROR       */
	.long	0x40000000 | (INT_LVL_SCI <<4)	/* 0x500: SCI_RX             */
	.long	0x40000000 | (INT_LVL_SCI <<4)	/* 0x520: SCI_TX             */
	.long	0x40000000 | (INT_LVL_SCI <<4)	/* 0x540: SCI_TX_ERROR       */
	.long	0x400000f0			/* 0x560: WDT_INTERVAL_TIMER */
	.long	0x400000f0			/* 0x580: BSC_REFRESH_CMI    */
	.long	0x400000f0			/* 0x5a0: BSC_REFRESH_OVF    */
	.long	0x400000f0			/* 0x5c0: (reserved)         */
	.long	0x400000f0			/* 0x5e0: (reserved)         */
	.long	0x400000f0			/* 0x600: Hitachi-UDI	     */
	.long	0x400000f0			/* 0x620: (reserved)         */
	.long	0x400000f0			/* 0x640: DMAC DMTE0         */
	.long	0x400000f0			/* 0x660: DMAC DMTE1         */
	.long	0x400000f0			/* 0x680: DMAC DMTE2         */
	.long	0x400000f0			/* 0x6a0: DMAC DMTE3         */
	.long	0x400000f0			/* 0x6c0: DMAC DMAE          */
	.long	0x400000f0			/* 0x6e0: (reserved)         */
	.long	0x40000000 | (INT_LVL_SCIF <<4)	/* 0x700: SCIF_RX_ERROR      */
	.long	0x40000000 | (INT_LVL_SCIF <<4)	/* 0x720: SCIF_RX            */
	.long	0x40000000 | (INT_LVL_SCIF <<4)	/* 0x740: SCIF_TX            */
	.long	0x40000000 | (INT_LVL_SCIF <<4)	/* 0x760: SCIF_TX_ERROR      */
	.long	0x400000f0			/* 0x780: (reserved)         */
	.long	0x400000f0			/* 0x7a0: (reserved)         */
	.long	0x400000f0			/* 0x7c0: (reserved)         */
	.long	0x400000f0			/* 0x7e0: (reserved)         */
	.long	0x400000f0			/* 0x800: FPU disable        */
	.long	0x400000f0			/* 0x820: Slot FPU disable   */
	.long	0x400000f0			/* 0x840: (reserved)         */
	.long	0x400000f0			/* 0x860: (reserved)         */
	.long	0x400000f0			/* 0x880: (reserved)         */
	.long	0x400000f0			/* 0x8a0: (reserved)         */
	.long	0x400000f0			/* 0x8c0: (reserved)         */
	.long	0x400000f0			/* 0x8e0: (reserved)         */
	.long	0x400000f0			/* 0x900: (reserved)         */
	.long	0x400000f0			/* 0x920: (reserved)         */
	.long	0x400000f0			/* 0x940: (reserved)         */
	.long	0x400000f0			/* 0x960: (reserved)         */
	.long	0x400000f0			/* 0x980: (reserved)         */
	.long	0x400000f0			/* 0x9a0: (reserved)         */
	.long	0x400000f0			/* 0x9c0: (reserved)         */
	.long	0x400000f0			/* 0x9e0: (reserved)         */
	.long	0x40000000 |(INT_LVL_PCISERR<<4)/* 0xa00: PCI SERR           */
	.long	0x40000000 |(INT_LVL_PCIDMA3<<4)/* 0xa20: PCIC DMA3 xfer end */
	.long	0x40000000 |(INT_LVL_PCIDMA2<<4)/* 0xa40: PCIC DMA2 xfer end */
	.long	0x40000000 |(INT_LVL_PCIDMA1<<4)/* 0xa60: PCIC DMA1 xfer end */
	.long	0x40000000 |(INT_LVL_PCIDMA0<<4)/* 0xa80: PCIC DMA2 xfer end */
	.long	0x40000000 |(INT_LVL_PCIPWON<<4)/* 0xaa0: PCIC Pwr. on req.  */
	.long	0x40000000 |(INT_LVL_PCIPWDN<<4)/* 0xac0: PCIC Pwr. down req */
	.long	0x40000000 |(INT_LVL_PCIERR <<4)/* 0xae0: PCIC error         */
	.long	0x400000f0			/* 0xb00: TMU3_UNDERFLOW     */
	.long	0x400000f0			/* 0xb20: (reserved)         */
	.long	0x400000f0			/* 0xb40: (reserved)         */
	.long	0x400000f0			/* 0xb60: (reserved)         */
	.long	0x400000f0			/* 0xb80: TMU4_UNDERFLOW     */

intPrioTableEnd:

			.align	2
			.type	_intPrioTableSize,@object
			.size	_intPrioTableSize,4
_intPrioTableSize:	.long	intPrioTableEnd - _intPrioTable
